@@ -38,10 +38,12 @@
 #include "gdata-types.h"
 #include "gdata-service.h"
 #include "gdata-private.h"
+#include "gdata-comparable.h"
 #include "atom/gdata-category.h"
 #include "atom/gdata-link.h"
 #include "atom/gdata-author.h"
 
+static void gdata_entry_constructed (GObject *object);
 static void gdata_entry_dispose (GObject *object);
 static void gdata_entry_finalize (GObject *object);
 static void gdata_entry_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec);
@@ -52,19 +54,25 @@ static gboolean post_parse_xml (GDataParsable *parsable, gpointer user_data, GEr
 static void pre_get_xml (GDataParsable *parsable, GString *xml_string);
 static void get_xml (GDataParsable *parsable, GString *xml_string);
 static void get_namespaces (GDataParsable *parsable, GHashTable *namespaces);
+static gchar *get_entry_uri (const gchar *id) G_GNUC_WARN_UNUSED_RESULT;
 
 struct _GDataEntryPrivate {
 	gchar *title;
 	gchar *summary;
 	gchar *id;
 	gchar *etag;
-	GTimeVal updated;
-	GTimeVal published;
+	gint64 updated;
+	gint64 published;
 	GList *categories; /* GDataCategory */
 	gchar *content;
+	gboolean content_is_uri;
 	GList *links; /* GDataLink */
 	GList *authors; /* GDataAuthor */
 	gchar *rights;
+
+	/* Batch processing data */
+	GDataBatchOperationType batch_operation_type;
+	guint batch_id;
 };
 
 enum {
@@ -76,11 +84,11 @@ enum {
 	PROP_PUBLISHED,
 	PROP_CONTENT,
 	PROP_IS_INSERTED,
-	PROP_RIGHTS
+	PROP_RIGHTS,
+	PROP_CONTENT_URI
 };
 
 G_DEFINE_TYPE (GDataEntry, gdata_entry, GDATA_TYPE_PARSABLE)
-#define GDATA_ENTRY_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GDATA_TYPE_ENTRY, GDataEntryPrivate))
 
 static void
 gdata_entry_class_init (GDataEntryClass *klass)
@@ -90,6 +98,7 @@ gdata_entry_class_init (GDataEntryClass *klass)
 
 	g_type_class_add_private (klass, sizeof (GDataEntryPrivate));
 
+	gobject_class->constructed = gdata_entry_constructed;
 	gobject_class->get_property = gdata_entry_get_property;
 	gobject_class->set_property = gdata_entry_set_property;
 	gobject_class->dispose = gdata_entry_dispose;
@@ -103,49 +112,51 @@ gdata_entry_class_init (GDataEntryClass *klass)
 	parsable_class->get_namespaces = get_namespaces;
 	parsable_class->element_name = "entry";
 
+	klass->get_entry_uri = get_entry_uri;
+
 	/**
 	 * GDataEntry:title:
 	 *
 	 * A human-readable title for the entry.
 	 *
-	 * For more information, see the <ulink type="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.title">
-	 * Atom specification</ulink>.
+	 * For more information, see the <ulink type="http"
+	 * url="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.title">Atom specification</ulink>.
 	 **/
 	g_object_class_install_property (gobject_class, PROP_TITLE,
-				g_param_spec_string ("title",
-					"Title", "A human-readable title for the entry.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("title",
+	                                                      "Title", "A human-readable title for the entry.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataEntry:summary:
 	 *
 	 * A short summary, abstract, or excerpt of the entry.
 	 *
-	 * For more information, see the <ulink type="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.summary">
-	 * Atom specification</ulink>.
+	 * For more information, see the <ulink type="http"
+	 * url="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.summary">Atom specification</ulink>.
 	 *
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_SUMMARY,
-				g_param_spec_string ("summary",
-					"Summary", "A short summary, abstract, or excerpt of the entry.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("summary",
+	                                                      "Summary", "A short summary, abstract, or excerpt of the entry.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataEntry:id:
 	 *
 	 * A permanent, universally unique identifier for the entry, in IRI form.
 	 *
-	 * For more information, see the <ulink type="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.id">
+	 * For more information, see the <ulink type="http" url="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.id">
 	 * Atom specification</ulink>.
 	 **/
 	g_object_class_install_property (gobject_class, PROP_ID,
-				g_param_spec_string ("id",
-					"ID", "A permanent, universally unique identifier for the entry, in IRI form.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("id",
+	                                                      "ID", "A permanent, universally unique identifier for the entry, in IRI form.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataEntry:etag:
@@ -153,58 +164,74 @@ gdata_entry_class_init (GDataEntryClass *klass)
 	 * An identifier for a particular version of the entry. This changes every time the entry on the server changes, and can be used
 	 * for conditional retrieval and locking.
 	 *
-	 * For more information, see the <ulink type="http://code.google.com/apis/gdata/docs/2.0/reference.html#ResourceVersioning">
+	 * For more information, see the <ulink type="http" url="http://code.google.com/apis/gdata/docs/2.0/reference.html#ResourceVersioning">
 	 * GData specification</ulink>.
 	 *
 	 * Since: 0.2.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_ETAG,
-				g_param_spec_string ("etag",
-					"ETag", "An identifier for a particular version of the entry.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("etag",
+	                                                      "ETag", "An identifier for a particular version of the entry.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataEntry:updated:
 	 *
-	 * The date and time the entry was most recently updated in a significant way.
+	 * The date and time when the entry was most recently updated significantly.
 	 *
-	 * For more information, see the <ulink type="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.updated">
-	 * Atom specification</ulink>.
+	 * For more information, see the <ulink type="http"
+	 * url="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.updated">Atom specification</ulink>.
 	 **/
 	g_object_class_install_property (gobject_class, PROP_UPDATED,
-				g_param_spec_boxed ("updated",
-					"Updated", "The date and time the entry was most recently updated in a significant way.",
-					GDATA_TYPE_G_TIME_VAL,
-					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_int64 ("updated",
+	                                                     "Updated", "The date and time when the entry was most recently updated significantly.",
+	                                                     -1, G_MAXINT64, -1,
+	                                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataEntry:published:
 	 *
 	 * The date and time the entry was first published or made available.
 	 *
-	 * For more information, see the <ulink type="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.published">
-	 * Atom specification</ulink>.
+	 * For more information, see the <ulink type="http"
+	 * url="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.published">Atom specification</ulink>.
 	 **/
 	g_object_class_install_property (gobject_class, PROP_PUBLISHED,
-				g_param_spec_boxed ("published",
-					"Published", "The date and time the entry was first published or made available.",
-					GDATA_TYPE_G_TIME_VAL,
-					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_int64 ("published",
+	                                                     "Published", "The date and time the entry was first published or made available.",
+	                                                     -1, G_MAXINT64, -1,
+	                                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataEntry:content:
 	 *
-	 * The content of the entry.
+	 * The content of the entry. This is mutually exclusive with #GDataEntry:content.
 	 *
-	 * For more information, see the <ulink type="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.published">
-	 * Atom specification</ulink>.
+	 * For more information, see the <ulink type="http"
+	 * url="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.content">Atom specification</ulink>.
 	 **/
 	g_object_class_install_property (gobject_class, PROP_CONTENT,
-				g_param_spec_string ("content",
-					"Content", "The content of the entry.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("content",
+	                                                      "Content", "The content of the entry.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+	/**
+	 * GDataEntry:content-uri:
+	 *
+	 * A URI pointing to the location of the content of the entry. This is mutually exclusive with #GDataEntry:content.
+	 *
+	 * For more information, see the
+	 * <ulink type="http" url="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.content">Atom specification</ulink>.
+	 *
+	 * Since: 0.7.0
+	 **/
+	g_object_class_install_property (gobject_class, PROP_CONTENT_URI,
+	                                 g_param_spec_string ("content-uri",
+	                                                      "Content URI", "A URI pointing to the location of the content of the entry.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataEntry:is-inserted:
@@ -213,31 +240,53 @@ gdata_entry_class_init (GDataEntryClass *klass)
 	 * %TRUE for entries returned from the server by queries. It is set to %TRUE when an entry is inserted using gdata_service_insert_entry().
 	 **/
 	g_object_class_install_property (gobject_class, PROP_IS_INSERTED,
-				g_param_spec_boolean ("is-inserted",
-					"Inserted?", "Whether the entry has been inserted on the server.",
-					FALSE,
-					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_boolean ("is-inserted",
+	                                                       "Inserted?", "Whether the entry has been inserted on the server.",
+	                                                       FALSE,
+	                                                       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
 	/**
 	 * GDataEntry:rights:
 	 *
 	 * The ownership rights pertaining to the entry.
 	 *
-	 * For more information, see the <ulink type="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.rights">
-	 * Atom specification</ulink>.
+	 * For more information, see the <ulink type="http"
+	 * url="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.rights">Atom specification</ulink>.
 	 *
 	 * Since: 0.5.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_RIGHTS,
-					 g_param_spec_string ("rights",
-							      "Rights", "The ownership rights pertaining to the entry.",
-							      NULL,
-							      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("rights",
+	                                                      "Rights", "The ownership rights pertaining to the entry.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
 gdata_entry_init (GDataEntry *self)
 {
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GDATA_TYPE_ENTRY, GDataEntryPrivate);
+	self->priv->updated = -1;
+	self->priv->published = -1;
+}
+
+static void
+gdata_entry_constructed (GObject *object)
+{
+	GDataEntryClass *klass = GDATA_ENTRY_GET_CLASS (object);
+	GObjectClass *parent_class = G_OBJECT_CLASS (gdata_entry_parent_class);
+
+	/* This can't be done in *_init() because the class properties haven't been properly set then */
+	if (klass->kind_term != NULL) {
+		/* Ensure we have the correct category/kind */
+		GDataCategory *category = gdata_category_new (klass->kind_term, "http://schemas.google.com/g/2005#kind", NULL);
+		gdata_entry_add_category (GDATA_ENTRY (object), category);
+		g_object_unref (category);
+	}
+
+	/* Chain up to the parent class */
+	if (parent_class->constructed != NULL)
+		parent_class->constructed (object);
 }
 
 static void
@@ -270,7 +319,7 @@ gdata_entry_dispose (GObject *object)
 static void
 gdata_entry_finalize (GObject *object)
 {
-	GDataEntryPrivate *priv = GDATA_ENTRY_GET_PRIVATE (object);
+	GDataEntryPrivate *priv = GDATA_ENTRY (object)->priv;
 
 	g_free (priv->title);
 	g_free (priv->summary);
@@ -286,7 +335,7 @@ gdata_entry_finalize (GObject *object)
 static void
 gdata_entry_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec)
 {
-	GDataEntryPrivate *priv = GDATA_ENTRY_GET_PRIVATE (object);
+	GDataEntryPrivate *priv = GDATA_ENTRY (object)->priv;
 
 	switch (property_id) {
 		case PROP_TITLE:
@@ -302,13 +351,16 @@ gdata_entry_get_property (GObject *object, guint property_id, GValue *value, GPa
 			g_value_set_string (value, priv->etag);
 			break;
 		case PROP_UPDATED:
-			g_value_set_boxed (value, &(priv->updated));
+			g_value_set_int64 (value, priv->updated);
 			break;
 		case PROP_PUBLISHED:
-			g_value_set_boxed (value, &(priv->published));
+			g_value_set_int64 (value, priv->published);
 			break;
 		case PROP_CONTENT:
-			g_value_set_string (value, priv->content);
+			g_value_set_string (value, (priv->content_is_uri == FALSE) ? priv->content : NULL);
+			break;
+		case PROP_CONTENT_URI:
+			g_value_set_string (value, (priv->content_is_uri == TRUE) ? priv->content : NULL);
 			break;
 		case PROP_IS_INSERTED:
 			g_value_set_boolean (value, gdata_entry_is_inserted (GDATA_ENTRY (object)));
@@ -346,6 +398,9 @@ gdata_entry_set_property (GObject *object, guint property_id, const GValue *valu
 		case PROP_CONTENT:
 			gdata_entry_set_content (self, g_value_get_string (value));
 			break;
+		case PROP_CONTENT_URI:
+			gdata_entry_set_content_uri (self, g_value_get_string (value));
+			break;
 		case PROP_RIGHTS:
 			gdata_entry_set_rights (self, g_value_get_string (value));
 			break;
@@ -359,10 +414,6 @@ gdata_entry_set_property (GObject *object, guint property_id, const GValue *valu
 static gboolean
 pre_parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *root_node, gpointer user_data, GError **error)
 {
-	g_return_val_if_fail (GDATA_IS_ENTRY (parsable), FALSE);
-	g_return_val_if_fail (doc != NULL, FALSE);
-	g_return_val_if_fail (root_node != NULL, FALSE);
-
 	/* Extract the ETag */
 	GDATA_ENTRY (parsable)->priv->etag = (gchar*) xmlGetProp (root_node, (xmlChar*) "etag");
 
@@ -372,84 +423,45 @@ pre_parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *root_node, gpointe
 static gboolean
 parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_data, GError **error)
 {
-	GDataEntry *self = GDATA_ENTRY (parsable);
+	gboolean success;
+	GDataEntryPrivate *priv = GDATA_ENTRY (parsable)->priv;
 
 	if (gdata_parser_is_namespace (node, "http://www.w3.org/2005/Atom") == TRUE) {
-		if (xmlStrcmp (node->name, (xmlChar*) "title") == 0) {
-			/* atom:title */
-			xmlChar *title = xmlNodeListGetString (doc, node->children, TRUE);
-
-			/* Title can be empty */
-			self->priv->title = (title != NULL) ? (gchar*) title : g_strdup ("");
-		} else if (xmlStrcmp (node->name, (xmlChar*) "id") == 0) {
-			/* atom:id */
-			g_free (self->priv->id);
-			self->priv->id = (gchar*) xmlNodeListGetString (doc, node->children, TRUE);
-		} else if (xmlStrcmp (node->name, (xmlChar*) "updated") == 0) {
-			/* atom:updated */
-			xmlChar *updated;
-
-			updated = xmlNodeListGetString (doc, node->children, TRUE);
-			if (g_time_val_from_iso8601 ((gchar*) updated, &(self->priv->updated)) == FALSE) {
-				/* Error */
-				gdata_parser_error_not_iso8601_format (node, (gchar*) updated, error);
-				xmlFree (updated);
-				return FALSE;
-			}
-			xmlFree (updated);
-		} else if (xmlStrcmp (node->name, (xmlChar*) "published") == 0) {
-			/* atom:published */
-			xmlChar *published;
-
-			published = xmlNodeListGetString (doc, node->children, TRUE);
-			if (g_time_val_from_iso8601 ((gchar*) published, &(self->priv->published)) == FALSE) {
-				/* Error */
-				gdata_parser_error_not_iso8601_format (node, (gchar*) published, error);
-				xmlFree (published);
-				return FALSE;
-			}
-			xmlFree (published);
-		} else if (xmlStrcmp (node->name, (xmlChar*) "category") == 0) {
-			/* atom:category */
-			GDataCategory *category = GDATA_CATEGORY (_gdata_parsable_new_from_xml_node (GDATA_TYPE_CATEGORY, doc, node, NULL, error));
-			if (category == NULL)
-				return FALSE;
-
-			self->priv->categories = g_list_prepend (self->priv->categories, category);
+		if (gdata_parser_string_from_element (node, "title", P_DEFAULT, &(priv->title), &success, error) == TRUE ||
+		    gdata_parser_string_from_element (node, "id", P_REQUIRED | P_NON_EMPTY | P_NO_DUPES, &(priv->id), &success, error) == TRUE ||
+		    gdata_parser_string_from_element (node, "summary", P_NONE, &(priv->summary), &success, error) == TRUE ||
+		    gdata_parser_string_from_element (node, "rights", P_NONE, &(priv->rights), &success, error) == TRUE ||
+		    gdata_parser_int64_from_element (node, "updated", P_REQUIRED | P_NO_DUPES, &(priv->updated), &success, error) == TRUE ||
+		    gdata_parser_int64_from_element (node, "published", P_REQUIRED | P_NO_DUPES, &(priv->published), &success, error) == TRUE ||
+		    gdata_parser_object_from_element_setter (node, "category", P_REQUIRED, GDATA_TYPE_CATEGORY,
+		                                             gdata_entry_add_category, parsable, &success, error) == TRUE ||
+		    gdata_parser_object_from_element_setter (node, "link", P_REQUIRED, GDATA_TYPE_LINK,
+		                                             gdata_entry_add_link, parsable, &success, error) == TRUE ||
+		    gdata_parser_object_from_element_setter (node, "author", P_REQUIRED, GDATA_TYPE_AUTHOR,
+		                                             gdata_entry_add_author, parsable, &success, error) == TRUE) {
+			return success;
 		} else if (xmlStrcmp (node->name, (xmlChar*) "content") == 0) {
 			/* atom:content */
-			xmlChar *content = xmlNodeListGetString (doc, node->children, TRUE);
-			if (content == NULL)
-				content = xmlGetProp (node, (xmlChar*) "src");
-			self->priv->content = (gchar*) content;
-		} else if (xmlStrcmp (node->name, (xmlChar*) "link") == 0) {
-			/* atom:link */
-			GDataLink *link = GDATA_LINK (_gdata_parsable_new_from_xml_node (GDATA_TYPE_LINK, doc, node, NULL, error));
-			if (link == NULL)
-				return FALSE;
+			priv->content = (gchar*) xmlGetProp (node, (xmlChar*) "src");
+			priv->content_is_uri = TRUE;
 
-			self->priv->links = g_list_prepend (self->priv->links, link);
-		} else if (xmlStrcmp (node->name, (xmlChar*) "author") == 0) {
-			/* atom:author */
-			GDataAuthor *author = GDATA_AUTHOR (_gdata_parsable_new_from_xml_node (GDATA_TYPE_AUTHOR, doc, node, NULL, error));
-			if (author == NULL)
-				return FALSE;
+			if (priv->content == NULL) {
+				priv->content = (gchar*) xmlNodeListGetString (doc, node->children, TRUE);
+				priv->content_is_uri = FALSE;
+			}
 
-			self->priv->authors = g_list_prepend (self->priv->authors, author);
-		} else if (xmlStrcmp (node->name, (xmlChar*) "summary") == 0) {
-			/* atom:summary */
-			self->priv->summary = (gchar*) xmlNodeListGetString (doc, node->children, TRUE);
-		} else if (xmlStrcmp (node->name, (xmlChar*) "rights") == 0) {
-			/* atom:rights */
-			self->priv->rights = (gchar*) xmlNodeListGetString (doc, node->children, TRUE);
-		} else {
-			return GDATA_PARSABLE_CLASS (gdata_entry_parent_class)->parse_xml (parsable, doc, node, user_data, error);
+			return TRUE;
+		}
+	} else if (gdata_parser_is_namespace (node, "http://schemas.google.com/gdata/batch") == TRUE) {
+		if (xmlStrcmp (node->name, (xmlChar*) "id") == 0 ||
+		    xmlStrcmp (node->name, (xmlChar*) "status") == 0 ||
+		    xmlStrcmp (node->name, (xmlChar*) "operation") == 0) {
+			/* Ignore batch operation elements; they're handled in GDataBatchFeed */
+			return TRUE;
 		}
-	} else {
-		return GDATA_PARSABLE_CLASS (gdata_entry_parent_class)->parse_xml (parsable, doc, node, user_data, error);
 	}
 
-	return TRUE;
+	return GDATA_PARSABLE_CLASS (gdata_entry_parent_class)->parse_xml (parsable, doc, node, user_data, error);
 }
 
 static gboolean
@@ -480,7 +492,7 @@ pre_get_xml (GDataParsable *parsable, GString *xml_string)
 	GDataEntryPrivate *priv = GDATA_ENTRY (parsable)->priv;
 
 	/* Add the entry's ETag, if available */
-	if (priv->etag != NULL)
+	if (gdata_entry_get_etag (GDATA_ENTRY (parsable)) != NULL)
 		g_string_append_printf (xml_string, " gd:etag='%s'", priv->etag);
 }
 
@@ -495,14 +507,14 @@ get_xml (GDataParsable *parsable, GString *xml_string)
 	if (priv->id != NULL)
 		g_string_append_printf (xml_string, "<id>%s</id>", priv->id);
 
-	if (priv->updated.tv_sec != 0 || priv->updated.tv_usec != 0) {
-		gchar *updated = g_time_val_to_iso8601 (&(priv->updated));
+	if (priv->updated != -1) {
+		gchar *updated = gdata_parser_int64_to_iso8601 (priv->updated);
 		g_string_append_printf (xml_string, "<updated>%s</updated>", updated);
 		g_free (updated);
 	}
 
-	if (priv->published.tv_sec != 0 || priv->published.tv_usec != 0) {
-		gchar *published = g_time_val_to_iso8601 (&(priv->published));
+	if (priv->published != -1) {
+		gchar *published = gdata_parser_int64_to_iso8601 (priv->published);
 		g_string_append_printf (xml_string, "<published>%s</published>", published);
 		g_free (published);
 	}
@@ -513,8 +525,12 @@ get_xml (GDataParsable *parsable, GString *xml_string)
 	if (priv->rights != NULL)
 		gdata_parser_string_append_escaped (xml_string, "<rights>", priv->rights, "</rights>");
 
-	if (priv->content != NULL)
-		gdata_parser_string_append_escaped (xml_string, "<content type='text'>", priv->content, "</content>");
+	if (priv->content != NULL) {
+		if (priv->content_is_uri == TRUE)
+			gdata_parser_string_append_escaped (xml_string, "<content type='text/plain' src='", priv->content, "'/>");
+		else
+			gdata_parser_string_append_escaped (xml_string, "<content type='text'>", priv->content, "</content>");
+	}
 
 	for (categories = priv->categories; categories != NULL; categories = categories->next)
 		_gdata_parsable_get_xml (GDATA_PARSABLE (categories->data), xml_string, FALSE);
@@ -524,12 +540,48 @@ get_xml (GDataParsable *parsable, GString *xml_string)
 
 	for (authors = priv->authors; authors != NULL; authors = authors->next)
 		_gdata_parsable_get_xml (GDATA_PARSABLE (authors->data), xml_string, FALSE);
+
+	/* Batch operation data */
+	if (priv->batch_id != 0) {
+		const gchar *batch_op;
+
+		switch (priv->batch_operation_type) {
+			case GDATA_BATCH_OPERATION_QUERY:
+				batch_op = "query";
+				break;
+			case GDATA_BATCH_OPERATION_INSERTION:
+				batch_op = "insert";
+				break;
+			case GDATA_BATCH_OPERATION_UPDATE:
+				batch_op = "update";
+				break;
+			case GDATA_BATCH_OPERATION_DELETION:
+				batch_op = "delete";
+				break;
+			default:
+				g_assert_not_reached ();
+				break;
+		}
+
+		g_string_append_printf (xml_string, "<batch:id>%u</batch:id><batch:operation type='%s'/>", priv->batch_id, batch_op);
+	}
 }
 
 static void
 get_namespaces (GDataParsable *parsable, GHashTable *namespaces)
 {
 	g_hash_table_insert (namespaces, (gchar*) "gd", (gchar*) "http://schemas.google.com/g/2005");
+
+	if (GDATA_ENTRY (parsable)->priv->batch_id != 0)
+		g_hash_table_insert (namespaces, (gchar*) "batch", (gchar*) "http://schemas.google.com/gdata/batch");
+}
+
+static gchar *
+get_entry_uri (const gchar *id)
+{
+	/* We assume the entry ID is also its entry URI; subclasses can override this
+	 * if the service they implement has a convoluted API */
+	return g_strdup (id);
 }
 
 /**
@@ -643,38 +695,65 @@ gdata_entry_get_id (GDataEntry *self)
 const gchar *
 gdata_entry_get_etag (GDataEntry *self)
 {
+	gchar *etag;
+
 	g_return_val_if_fail (GDATA_IS_ENTRY (self), NULL);
-	return self->priv->etag;
+
+	/* We have to check if the property's set since GDataAccessRule overrides it and sets it to always be NULL (since ACL entries don't support
+	 * ETags, for some reason). */
+	g_object_get (G_OBJECT (self), "etag", &etag, NULL);
+	if (etag != NULL) {
+		g_free (etag);
+		return self->priv->etag;
+	}
+
+	return NULL;
 }
 
 /**
  * gdata_entry_get_updated:
  * @self: a #GDataEntry
- * @updated: (out caller-allocates): a #GTimeVal
  *
- * Puts the time the entry was last updated into @updated.
+ * Gets the time the entry was last updated.
+ *
+ * Return value: the UNIX timestamp for the last update of the entry
  **/
+gint64
+gdata_entry_get_updated (GDataEntry *self)
+{
+	g_return_val_if_fail (GDATA_IS_ENTRY (self), -1);
+	return self->priv->updated;
+}
+
+/*
+ * _gdata_entry_set_updated:
+ * @self: a #GDataEntry
+ * @updated: the new updated value
+ *
+ * Sets the value of the #GDataEntry:updated property to @updated.
+ *
+ * Since: 0.6.0
+ */
 void
-gdata_entry_get_updated (GDataEntry *self, GTimeVal *updated)
+_gdata_entry_set_updated (GDataEntry *self, gint64 updated)
 {
 	g_return_if_fail (GDATA_IS_ENTRY (self));
-	g_return_if_fail (updated != NULL);
-	*updated = self->priv->updated;
+	self->priv->updated = updated;
 }
 
 /**
  * gdata_entry_get_published:
  * @self: a #GDataEntry
- * @published: (out caller-allocates): a #GTimeVal
  *
- * Puts the time the entry was originally published into @published.
+ * Gets the time the entry was originally published.
+ *
+ * Return value: the UNIX timestamp for the original publish time of the entry
  **/
-void
-gdata_entry_get_published (GDataEntry *self, GTimeVal *published)
+gint64
+gdata_entry_get_published (GDataEntry *self)
 {
-	g_return_if_fail (GDATA_IS_ENTRY (self));
-	g_return_if_fail (published != NULL);
-	*published = self->priv->published;
+	g_return_val_if_fail (GDATA_IS_ENTRY (self), -1);
+	return self->priv->published;
 }
 
 /**
@@ -692,7 +771,18 @@ gdata_entry_add_category (GDataEntry *self, GDataCategory *category)
 	g_return_if_fail (GDATA_IS_ENTRY (self));
 	g_return_if_fail (GDATA_IS_CATEGORY (category));
 
-	if (g_list_find_custom (self->priv->categories, category, (GCompareFunc) gdata_category_compare) == NULL)
+	/* Check to see if it's a kind category and if it matches the entry's predetermined kind */
+	if (g_strcmp0 (gdata_category_get_scheme (category), "http://schemas.google.com/g/2005#kind") == 0) {
+		GDataEntryClass *klass = GDATA_ENTRY_GET_CLASS (self);
+
+		if (klass->kind_term != NULL && g_strcmp0 (gdata_category_get_term (category), klass->kind_term) != 0) {
+			g_warning ("Adding a kind category term, '%s', to an entry of kind '%s'.",
+			           gdata_category_get_term (category), klass->kind_term);
+		}
+	}
+
+	/* Add the category if we don't already have it */
+	if (g_list_find_custom (self->priv->categories, category, (GCompareFunc) gdata_comparable_compare) == NULL)
 		self->priv->categories = g_list_prepend (self->priv->categories, g_object_ref (category));
 }
 
@@ -713,11 +803,29 @@ gdata_entry_get_categories (GDataEntry *self)
 	return self->priv->categories;
 }
 
+/**
+ * gdata_entry_get_authors:
+ * @self: a #GDataEntry
+ *
+ * Gets a list of the #GDataAuthor<!-- -->s for this entry.
+ *
+ * Return value: (element-type GData.Author) (transfer none): a #GList of #GDataAuthor<!-- -->s
+ *
+ * Since: 0.7.0
+ **/
+GList *
+gdata_entry_get_authors (GDataEntry *self)
+{
+	g_return_val_if_fail (GDATA_IS_ENTRY (self), NULL);
+	return self->priv->authors;
+}
+
 /**
  * gdata_entry_get_content:
  * @self: a #GDataEntry
  *
- * Returns the textual content in this entry.
+ * Returns the textual content in this entry. If the content in this entry is pointed to by a URI, %NULL will be returned; the content URI will be
+ * returned by gdata_entry_get_content_uri().
  *
  * Return value: the entry's content, or %NULL
  **/
@@ -725,7 +833,7 @@ const gchar *
 gdata_entry_get_content (GDataEntry *self)
 {
 	g_return_val_if_fail (GDATA_IS_ENTRY (self), NULL);
-	return self->priv->content;
+	return (self->priv->content_is_uri == FALSE) ? self->priv->content : NULL;
 }
 
 /**
@@ -733,7 +841,7 @@ gdata_entry_get_content (GDataEntry *self)
  * @self: a #GDataEntry
  * @content: (allow-none): the new content for the entry, or %NULL
  *
- * Sets the entry's content to @content.
+ * Sets the entry's content to @content. This unsets #GDataEntry:content-uri.
  **/
 void
 gdata_entry_set_content (GDataEntry *self, const gchar *content)
@@ -742,33 +850,80 @@ gdata_entry_set_content (GDataEntry *self, const gchar *content)
 
 	g_free (self->priv->content);
 	self->priv->content = g_strdup (content);
+	self->priv->content_is_uri = FALSE;
+
+	g_object_freeze_notify (G_OBJECT (self));
 	g_object_notify (G_OBJECT (self), "content");
+	g_object_notify (G_OBJECT (self), "content-uri");
+	g_object_thaw_notify (G_OBJECT (self));
+}
+
+/**
+ * gdata_entry_get_content_uri:
+ * @self: a #GDataEntry
+ *
+ * Returns a URI pointing to the content of this entry. If the content in this entry is stored directly, %NULL will be returned; the content will be
+ * returned by gdata_entry_get_content().
+ *
+ * Return value: a URI pointing to the entry's content, or %NULL
+ *
+ * Since: 0.7.0
+ **/
+const gchar *
+gdata_entry_get_content_uri (GDataEntry *self)
+{
+	g_return_val_if_fail (GDATA_IS_ENTRY (self), NULL);
+	return (self->priv->content_is_uri == TRUE) ? self->priv->content : NULL;
+}
+
+/**
+ * gdata_entry_set_content_uri:
+ * @self: a #GDataEntry
+ * @content_uri: (allow-none): the new URI pointing to the content for the entry, or %NULL
+ *
+ * Sets the URI pointing to the entry's content to @content. This unsets #GDataEntry:content.
+ *
+ * Since: 0.7.0
+ **/
+void
+gdata_entry_set_content_uri (GDataEntry *self, const gchar *content_uri)
+{
+	g_return_if_fail (GDATA_IS_ENTRY (self));
+
+	g_free (self->priv->content);
+	self->priv->content = g_strdup (content_uri);
+	self->priv->content_is_uri = TRUE;
+
+	g_object_freeze_notify (G_OBJECT (self));
+	g_object_notify (G_OBJECT (self), "content");
+	g_object_notify (G_OBJECT (self), "content-uri");
+	g_object_thaw_notify (G_OBJECT (self));
 }
 
 /**
  * gdata_entry_add_link:
  * @self: a #GDataEntry
- * @link: a #GDataLink to add
+ * @_link: a #GDataLink to add
  *
- * Adds @link to the list of links in the given #GDataEntry and increments its reference count.
+ * Adds @_link to the list of links in the given #GDataEntry and increments its reference count.
  *
  * Duplicate links will not be added to the list.
  **/
 void
-gdata_entry_add_link (GDataEntry *self, GDataLink *link)
+gdata_entry_add_link (GDataEntry *self, GDataLink *_link)
 {
 	/* TODO: More link API */
 	g_return_if_fail (GDATA_IS_ENTRY (self));
-	g_return_if_fail (GDATA_IS_LINK (link));
+	g_return_if_fail (GDATA_IS_LINK (_link));
 
-	if (g_list_find_custom (self->priv->links, link, (GCompareFunc) gdata_link_compare) == NULL)
-		self->priv->links = g_list_prepend (self->priv->links, g_object_ref (link));
+	if (g_list_find_custom (self->priv->links, _link, (GCompareFunc) gdata_comparable_compare) == NULL)
+		self->priv->links = g_list_prepend (self->priv->links, g_object_ref (_link));
 }
 
 static gint
-link_compare_cb (const GDataLink *link, const gchar *rel)
+link_compare_cb (const GDataLink *_link, const gchar *rel)
 {
-	return strcmp (gdata_link_get_relation_type ((GDataLink*) link), rel);
+	return strcmp (gdata_link_get_relation_type ((GDataLink*) _link), rel);
 }
 
 /**
@@ -845,9 +1000,9 @@ gdata_entry_add_author (GDataEntry *self, GDataAuthor *author)
 {
 	/* TODO: More author API */
 	g_return_if_fail (GDATA_IS_ENTRY (self));
-	g_return_if_fail (author != NULL);
+	g_return_if_fail (GDATA_IS_AUTHOR (author));
 
-	if (g_list_find_custom (self->priv->authors, author, (GCompareFunc) gdata_author_compare) == NULL)
+	if (g_list_find_custom (self->priv->authors, author, (GCompareFunc) gdata_comparable_compare) == NULL)
 		self->priv->authors = g_list_prepend (self->priv->authors, g_object_ref (author));
 }
 
@@ -864,8 +1019,7 @@ gdata_entry_is_inserted (GDataEntry *self)
 {
 	g_return_val_if_fail (GDATA_IS_ENTRY (self), FALSE);
 
-	if (self->priv->id != NULL ||
-	    self->priv->updated.tv_sec != 0 || self->priv->updated.tv_usec != 0)
+	if (self->priv->id != NULL || self->priv->updated != -1)
 		return TRUE;
 	return FALSE;
 }
@@ -905,3 +1059,22 @@ gdata_entry_set_rights (GDataEntry *self, const gchar *rights)
 	self->priv->rights = g_strdup (rights);
 	g_object_notify (G_OBJECT (self), "rights");
 }
+
+/*
+ * _gdata_entry_set_batch_data:
+ * @self: a #GDataEntry
+ * @id: the batch operation ID
+ * @type: the type of batch operation being performed on the #GDataEntry
+ *
+ * Sets the batch operation data needed when outputting the XML for a #GDataEntry to be put into a batch operation feed.
+ *
+ * Since: 0.6.0
+ */
+void
+_gdata_entry_set_batch_data (GDataEntry *self, guint id, GDataBatchOperationType type)
+{
+	g_return_if_fail (GDATA_IS_ENTRY (self));
+
+	self->priv->batch_id = id;
+	self->priv->batch_operation_type = type;
+}
