@@ -25,6 +25,10 @@
  *
  * #GDataCalendarCalendar is a subclass of #GDataEntry to represent a calendar from Google Calendar.
  *
+ * #GDataCalendarCalendar implements #GDataAccessHandler, meaning the access rules to it can be modified using that interface. As well as the
+ * access roles defined for the base #GDataAccessRule (e.g. %GDATA_ACCESS_ROLE_NONE), #GDataCalendarCalendar has its own, such as
+ * %GDATA_CALENDAR_ACCESS_ROLE_EDITOR and %GDATA_CALENDAR_ACCESS_ROLE_FREE_BUSY.
+ *
  * For more details of Google Calendar's GData API, see the <ulink type="http" url="http://code.google.com/apis/calendar/docs/2.0/reference.html">
  * online documentation</ulink>.
  **/
@@ -43,6 +47,7 @@
 #include "gdata-access-handler.h"
 
 static void gdata_calendar_calendar_access_handler_init (GDataAccessHandlerIface *iface);
+static GObject *gdata_calendar_calendar_constructor (GType type, guint n_construct_params, GObjectConstructParam *construct_params);
 static void gdata_calendar_calendar_finalize (GObject *object);
 static void gdata_calendar_calendar_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec);
 static void gdata_calendar_calendar_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec);
@@ -58,7 +63,7 @@ struct _GDataCalendarCalendarPrivate {
 	gboolean is_selected;
 	gchar *access_level;
 
-	GTimeVal edited;
+	gint64 edited;
 };
 
 enum {
@@ -72,17 +77,18 @@ enum {
 };
 
 G_DEFINE_TYPE_WITH_CODE (GDataCalendarCalendar, gdata_calendar_calendar, GDATA_TYPE_ENTRY,
-			 G_IMPLEMENT_INTERFACE (GDATA_TYPE_ACCESS_HANDLER, gdata_calendar_calendar_access_handler_init))
-#define GDATA_CALENDAR_CALENDAR_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GDATA_TYPE_CALENDAR_CALENDAR, GDataCalendarCalendarPrivate))
+                         G_IMPLEMENT_INTERFACE (GDATA_TYPE_ACCESS_HANDLER, gdata_calendar_calendar_access_handler_init))
 
 static void
 gdata_calendar_calendar_class_init (GDataCalendarCalendarClass *klass)
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 	GDataParsableClass *parsable_class = GDATA_PARSABLE_CLASS (klass);
+	GDataEntryClass *entry_class = GDATA_ENTRY_CLASS (klass);
 
 	g_type_class_add_private (klass, sizeof (GDataCalendarCalendarPrivate));
 
+	gobject_class->constructor = gdata_calendar_calendar_constructor;
 	gobject_class->set_property = gdata_calendar_calendar_set_property;
 	gobject_class->get_property = gdata_calendar_calendar_get_property;
 	gobject_class->finalize = gdata_calendar_calendar_finalize;
@@ -91,27 +97,30 @@ gdata_calendar_calendar_class_init (GDataCalendarCalendarClass *klass)
 	parsable_class->get_xml = get_xml;
 	parsable_class->get_namespaces = get_namespaces;
 
+	entry_class->kind_term = "http://schemas.google.com/gCal/2005#calendarmeta";
+
 	/**
 	 * GDataCalendarCalendar:timezone:
 	 *
-	 * The timezone in which the calendar's times are given.
+	 * The timezone in which the calendar's times are given. This is a timezone name in tz database notation: <ulink type="http"
+	 * url="http://en.wikipedia.org/wiki/Tz_database#Names_of_time_zones">reference</ulink>.
 	 **/
 	g_object_class_install_property (gobject_class, PROP_TIMEZONE,
-				g_param_spec_string ("timezone",
-					"Timezone", "The timezone in which the calendar's times are given.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("timezone",
+	                                                      "Timezone", "The timezone in which the calendar's times are given.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataCalendarCalendar:times-cleaned:
 	 *
-	 * The number of times the calendar has been completely cleared of events.
+	 * The number of times the calendar has been cleared of events.
 	 **/
 	g_object_class_install_property (gobject_class, PROP_TIMES_CLEANED,
-				g_param_spec_uint ("times-cleaned",
-					"Times cleaned", "The number of times the calendar has been completely cleared of events.",
-					0, G_MAXUINT, 0,
-					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_uint ("times-cleaned",
+	                                                    "Times cleaned", "The number of times the calendar has been cleared of events.",
+	                                                    0, G_MAXUINT, 0,
+	                                                    G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataCalendarCalendar:is-hidden:
@@ -121,10 +130,10 @@ gdata_calendar_calendar_class_init (GDataCalendarCalendarClass *klass)
 	 * Since: 0.2.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_IS_HIDDEN,
-				g_param_spec_boolean ("is-hidden",
-					"Hidden?", "Indicates whether the calendar is visible.",
-					FALSE,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_boolean ("is-hidden",
+	                                                       "Hidden?", "Indicates whether the calendar is visible.",
+	                                                       FALSE,
+	                                                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataCalendarCalendar:color:
@@ -132,10 +141,10 @@ gdata_calendar_calendar_class_init (GDataCalendarCalendarClass *klass)
 	 * The color used to highlight the calendar in the user's browser.
 	 **/
 	g_object_class_install_property (gobject_class, PROP_COLOR,
-				g_param_spec_boxed ("color",
-					"Color", "The color used to highlight the calendar in the user's browser.",
-					GDATA_TYPE_COLOR,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_boxed ("color",
+	                                                     "Color", "The color used to highlight the calendar in the user's browser.",
+	                                                     GDATA_TYPE_COLOR,
+	                                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataCalendarCalendar:is-selected:
@@ -145,21 +154,22 @@ gdata_calendar_calendar_class_init (GDataCalendarCalendarClass *klass)
 	 * Since: 0.2.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_IS_SELECTED,
-				g_param_spec_boolean ("is-selected",
-					"Selected?", "Indicates whether the calendar is selected.",
-					FALSE,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_boolean ("is-selected",
+	                                                       "Selected?", "Indicates whether the calendar is selected.",
+	                                                       FALSE,
+	                                                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataCalendarCalendar:access-level:
 	 *
-	 * Indicates what level of access the authenticated user has to the calendar.
+	 * Indicates the access level the current user has to the calendar. For example: %GDATA_CALENDAR_ACCESS_ROLE_READ or
+	 * %GDATA_CALENDAR_ACCESS_ROLE_FREE_BUSY. The "current user" is the one logged in with gdata_service_authenticate() or the guest user.
 	 **/
 	g_object_class_install_property (gobject_class, PROP_ACCESS_LEVEL,
-				g_param_spec_string ("access-level",
-					"Access level", "Indicates what level of access the authenticated user has to the calendar.",
-					NULL,
-					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("access-level",
+	                                                      "Access level", "Indicates the access level the current user has to the calendar.",
+	                                                      NULL,
+	                                                      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataCalendarCalendar:edited:
@@ -170,22 +180,16 @@ gdata_calendar_calendar_class_init (GDataCalendarCalendarClass *klass)
 	 * Atom Publishing Protocol specification</ulink>.
 	 **/
 	g_object_class_install_property (gobject_class, PROP_EDITED,
-				g_param_spec_boxed ("edited",
-					"Edited", "The last time the calendar was edited.",
-					GDATA_TYPE_G_TIME_VAL,
-					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
-}
-
-static void
-gdata_calendar_calendar_init (GDataCalendarCalendar *self)
-{
-	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GDATA_TYPE_CALENDAR_CALENDAR, GDataCalendarCalendarPrivate);
+	                                 g_param_spec_int64 ("edited",
+	                                                     "Edited", "The last time the calendar was edited.",
+	                                                     -1, G_MAXINT64, -1,
+	                                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 }
 
 static gboolean
 is_owner_rule (GDataAccessRule *rule)
 {
-	return (strcmp (gdata_access_rule_get_role (rule), "http://schemas.google.com/gCal/2005#owner") == 0) ? TRUE : FALSE;
+	return (strcmp (gdata_access_rule_get_role (rule), GDATA_CALENDAR_ACCESS_ROLE_OWNER) == 0) ? TRUE : FALSE;
 }
 
 static void
@@ -194,10 +198,38 @@ gdata_calendar_calendar_access_handler_init (GDataAccessHandlerIface *iface)
 	iface->is_owner_rule = is_owner_rule;
 }
 
+static void
+gdata_calendar_calendar_init (GDataCalendarCalendar *self)
+{
+	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GDATA_TYPE_CALENDAR_CALENDAR, GDataCalendarCalendarPrivate);
+	self->priv->edited = -1;
+}
+
+static GObject *
+gdata_calendar_calendar_constructor (GType type, guint n_construct_params, GObjectConstructParam *construct_params)
+{
+	GObject *object;
+
+	/* Chain up to the parent class */
+	object = G_OBJECT_CLASS (gdata_calendar_calendar_parent_class)->constructor (type, n_construct_params, construct_params);
+
+	if (_gdata_parsable_is_constructed_from_xml (GDATA_PARSABLE (object)) == FALSE) {
+		GDataCalendarCalendarPrivate *priv = GDATA_CALENDAR_CALENDAR (object)->priv;
+		GTimeVal time_val;
+
+		/* Set the edited property to the current time (creation time). We don't do this in *_init() since that would cause
+		 * setting it from parse_xml() to fail (duplicate element). */
+		g_get_current_time (&time_val);
+		priv->edited = time_val.tv_sec;
+	}
+
+	return object;
+}
+
 static void
 gdata_calendar_calendar_finalize (GObject *object)
 {
-	GDataCalendarCalendarPrivate *priv = GDATA_CALENDAR_CALENDAR_GET_PRIVATE (object);
+	GDataCalendarCalendarPrivate *priv = GDATA_CALENDAR_CALENDAR (object)->priv;
 
 	g_free (priv->timezone);
 	g_free (priv->access_level);
@@ -209,7 +241,7 @@ gdata_calendar_calendar_finalize (GObject *object)
 static void
 gdata_calendar_calendar_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec)
 {
-	GDataCalendarCalendarPrivate *priv = GDATA_CALENDAR_CALENDAR_GET_PRIVATE (object);
+	GDataCalendarCalendarPrivate *priv = GDATA_CALENDAR_CALENDAR (object)->priv;
 
 	switch (property_id) {
 		case PROP_TIMEZONE:
@@ -231,7 +263,7 @@ gdata_calendar_calendar_get_property (GObject *object, guint property_id, GValue
 			g_value_set_string (value, priv->access_level);
 			break;
 		case PROP_EDITED:
-			g_value_set_boxed (value, &(priv->edited));
+			g_value_set_int64 (value, priv->edited);
 			break;
 		default:
 			/* We don't have any other property... */
@@ -265,26 +297,16 @@ gdata_calendar_calendar_set_property (GObject *object, guint property_id, const
 	}
 }
 
-/**
- * gdata_calendar_calendar_new:
- * @id: the calendar's ID, or %NULL
- *
- * Creates a new #GDataCalendarCalendar with the given ID and default properties.
- *
- * Return value: a new #GDataCalendarCalendar; unref with g_object_unref()
- **/
-GDataCalendarCalendar *
-gdata_calendar_calendar_new (const gchar *id)
-{
-	return g_object_new (GDATA_TYPE_CALENDAR_CALENDAR, "id", id, NULL);
-}
-
 static gboolean
 parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_data, GError **error)
 {
+	gboolean success;
 	GDataCalendarCalendar *self = GDATA_CALENDAR_CALENDAR (parsable);
 
-	if (gdata_parser_is_namespace (node, "http://schemas.google.com/gCal/2005") == TRUE) {
+	if (gdata_parser_is_namespace (node, "http://www.w3.org/2007/app") == TRUE &&
+	    gdata_parser_int64_from_element (node, "edited", P_REQUIRED | P_NO_DUPES, &(self->priv->edited), &success, error) == TRUE) {
+		return success;
+	} else if (gdata_parser_is_namespace (node, "http://schemas.google.com/gCal/2005") == TRUE) {
 		if (xmlStrcmp (node->name, (xmlChar*) "timezone") == 0) {
 			/* gCal:timezone */
 			xmlChar *_timezone = xmlGetProp (node, (xmlChar*) "value");
@@ -300,11 +322,8 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 			xmlFree (times_cleaned);
 		} else if (xmlStrcmp (node->name, (xmlChar*) "hidden") == 0) {
 			/* gCal:hidden */
-			xmlChar *hidden = xmlGetProp (node, (xmlChar*) "value");
-			if (hidden == NULL)
-				return gdata_parser_error_required_property_missing (node, "value", error);
-			gdata_calendar_calendar_set_is_hidden (self, (xmlStrcmp (hidden, (xmlChar*) "true") == 0) ? TRUE : FALSE);
-			xmlFree (hidden);
+			if (gdata_parser_boolean_from_property (node, "value", &(self->priv->is_hidden), -1, error) == FALSE)
+				return FALSE;
 		} else if (xmlStrcmp (node->name, (xmlChar*) "color") == 0) {
 			/* gCal:color */
 			xmlChar *value;
@@ -316,12 +335,14 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 			if (gdata_color_from_hexadecimal ((gchar*) value, &colour) == FALSE) {
 				/* Error */
 				g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR,
-					     /* Translators: the first parameter is the name of an XML element (including the angle brackets ("<" and ">"),
-					      * and the second parameter is the erroneous value (which was not in hexadecimal RGB format).
-					      *
-					      * For example:
-					      *  The content of a <entry/gCal:color> element ("00FG56") was not in hexadecimal RGB format. */
-					     _("The content of a %s element (\"%s\") was not in hexadecimal RGB format."), "<entry/gCal:color>", value);
+				             /* Translators: the first parameter is the name of an XML element (including the angle brackets
+				              * ("<" and ">"), and the second parameter is the erroneous value (which was not in hexadecimal
+				              * RGB format).
+				              *
+				              * For example:
+				              *  The content of a <entry/gCal:color> element ("00FG56") was not in hexadecimal RGB format. */
+				             _("The content of a %s element (\"%s\") was not in hexadecimal RGB format."),
+				             "<entry/gCal:color>", value);
 				xmlFree (value);
 
 				return FALSE;
@@ -331,11 +352,8 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 			xmlFree (value);
 		} else if (xmlStrcmp (node->name, (xmlChar*) "selected") == 0) {
 			/* gCal:selected */
-			xmlChar *selected = xmlGetProp (node, (xmlChar*) "value");
-			if (selected == NULL)
-				return gdata_parser_error_required_property_missing (node, "value", error);
-			gdata_calendar_calendar_set_is_selected (self, (xmlStrcmp (selected, (xmlChar*) "true") == 0) ? TRUE : FALSE);
-			xmlFree (selected);
+			if (gdata_parser_boolean_from_property (node, "value", &(self->priv->is_selected), -1, error) == FALSE)
+				return FALSE;
 		} else if (xmlStrcmp (node->name, (xmlChar*) "accesslevel") == 0) {
 			/* gCal:accesslevel */
 			self->priv->access_level = (gchar*) xmlGetProp (node, (xmlChar*) "value");
@@ -344,17 +362,6 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 		} else {
 			return GDATA_PARSABLE_CLASS (gdata_calendar_calendar_parent_class)->parse_xml (parsable, doc, node, user_data, error);
 		}
-	} else if (gdata_parser_is_namespace (node, "http://www.w3.org/2007/app") == TRUE &&
-	           xmlStrcmp (node->name, (xmlChar*) "edited") == 0) {
-		/* app:edited */
-		xmlChar *edited = xmlNodeListGetString (doc, node->children, TRUE);
-		if (g_time_val_from_iso8601 ((gchar*) edited, &(self->priv->edited)) == FALSE) {
-			/* Error */
-			gdata_parser_error_not_iso8601_format (node, (gchar*) edited, error);
-			xmlFree (edited);
-			return FALSE;
-		}
-		xmlFree (edited);
 	} else {
 		return GDATA_PARSABLE_CLASS (gdata_calendar_calendar_parent_class)->parse_xml (parsable, doc, node, user_data, error);
 	}
@@ -400,6 +407,20 @@ get_namespaces (GDataParsable *parsable, GHashTable *namespaces)
 	g_hash_table_insert (namespaces, (gchar*) "app", (gchar*) "http://www.w3.org/2007/app");
 }
 
+/**
+ * gdata_calendar_calendar_new:
+ * @id: (allow-none): the calendar's ID, or %NULL
+ *
+ * Creates a new #GDataCalendarCalendar with the given ID and default properties.
+ *
+ * Return value: a new #GDataCalendarCalendar; unref with g_object_unref()
+ **/
+GDataCalendarCalendar *
+gdata_calendar_calendar_new (const gchar *id)
+{
+	return GDATA_CALENDAR_CALENDAR (g_object_new (GDATA_TYPE_CALENDAR_CALENDAR, "id", id, NULL));
+}
+
 /**
  * gdata_calendar_calendar_get_timezone:
  * @self: a #GDataCalendarCalendar
@@ -507,7 +528,7 @@ gdata_calendar_calendar_get_color (GDataCalendarCalendar *self, GDataColor *colo
  * Sets the #GDataCalendarCalendar:color property to @color.
  **/
 void
-gdata_calendar_calendar_set_color (GDataCalendarCalendar *self, GDataColor *color)
+gdata_calendar_calendar_set_color (GDataCalendarCalendar *self, const GDataColor *color)
 {
 	g_return_if_fail (GDATA_IS_CALENDAR_CALENDAR (self));
 	g_return_if_fail (color != NULL);
@@ -567,15 +588,14 @@ gdata_calendar_calendar_get_access_level (GDataCalendarCalendar *self)
 /**
  * gdata_calendar_calendar_get_edited:
  * @self: a #GDataCalendarCalendar
- * @edited: (out caller-allocates): a #GTimeVal
  *
- * Gets the #GDataCalendarCalendar:edited property and puts it in @edited. If the property is unset,
- * both fields in the #GTimeVal will be set to <code class="literal">0</code>.
+ * Gets the #GDataCalendarCalendar:edited property. If the property is unset, <code class="literal">-1</code> will be returned.
+ *
+ * Return value: the UNIX timestamp for the time the calendar was last edited, or <code class="literal">-1</code>
  **/
-void
-gdata_calendar_calendar_get_edited (GDataCalendarCalendar *self, GTimeVal *edited)
+gint64
+gdata_calendar_calendar_get_edited (GDataCalendarCalendar *self)
 {
-	g_return_if_fail (GDATA_IS_CALENDAR_CALENDAR (self));
-	g_return_if_fail (edited != NULL);
-	*edited = self->priv->edited;
+	g_return_val_if_fail (GDATA_IS_CALENDAR_CALENDAR (self), -1);
+	return self->priv->edited;
 }
