@@ -26,8 +26,12 @@
  * #GDataDocumentsEntry is a subclass of #GDataEntry to represent a Google Documents entry, which is then further subclassed
  * to give specific document types.
  *
- * For more details of Google Documents' GData API, see the <ulink type="http://code.google.com/apis/document/docs/2.0/developers_guide_protocol.html">
- * online documentation</ulink>.
+ * #GDataDocumentsEntry implements #GDataAccessHandler, meaning the access rules to it can be modified using that interface. As well as the
+ * access roles defined for the base #GDataAccessRule (e.g. %GDATA_ACCESS_ROLE_NONE), #GDataDocumentsEntry has its own, such as
+ * %GDATA_DOCUMENTS_ACCESS_ROLE_OWNER and %GDATA_DOCUMENTS_ACCESS_ROLE_READER.
+ *
+ * For more details of Google Documents' GData API, see the
+ * <ulink type="http" url="http://code.google.com/apis/document/docs/2.0/developers_guide_protocol.html">online documentation</ulink>.
  *
  * Since: 0.4.0
  **/
@@ -43,9 +47,9 @@
 #include "gdata-types.h"
 #include "gdata-private.h"
 #include "gdata-access-handler.h"
-#include "gdata-download-stream.h"
 
 static void gdata_documents_entry_access_handler_init (GDataAccessHandlerIface *iface);
+static GObject *gdata_documents_entry_constructor (GType type, guint n_construct_params, GObjectConstructParam *construct_params);
 static void gdata_documents_entry_finalize (GObject *object);
 static void gdata_entry_dispose (GObject *object);
 static void get_namespaces (GDataParsable *parsable, GHashTable *namespaces);
@@ -55,8 +59,8 @@ static void gdata_documents_entry_set_property (GObject *object, guint property_
 static gboolean parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_data, GError **error);
 
 struct _GDataDocumentsEntryPrivate {
-	GTimeVal edited;
-	GTimeVal last_viewed;
+	gint64 edited;
+	gint64 last_viewed;
 	gchar *document_id;
 	gboolean writers_can_invite;
 	gboolean is_deleted;
@@ -72,9 +76,8 @@ enum {
 	PROP_WRITERS_CAN_INVITE
 };
 
-G_DEFINE_TYPE_WITH_CODE (GDataDocumentsEntry, gdata_documents_entry, GDATA_TYPE_ENTRY,
-			 G_IMPLEMENT_INTERFACE (GDATA_TYPE_ACCESS_HANDLER, gdata_documents_entry_access_handler_init))
-#define GDATA_DOCUMENTS_ENTRY_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GDATA_TYPE_DOCUMENTS_ENTRY, GDataDocumentsEntryPrivate))
+G_DEFINE_ABSTRACT_TYPE_WITH_CODE (GDataDocumentsEntry, gdata_documents_entry, GDATA_TYPE_ENTRY,
+                                  G_IMPLEMENT_INTERFACE (GDATA_TYPE_ACCESS_HANDLER, gdata_documents_entry_access_handler_init))
 
 static void
 gdata_documents_entry_class_init (GDataDocumentsEntryClass *klass)
@@ -84,6 +87,7 @@ gdata_documents_entry_class_init (GDataDocumentsEntryClass *klass)
 
 	g_type_class_add_private (klass, sizeof (GDataDocumentsEntryPrivate));
 
+	gobject_class->constructor = gdata_documents_entry_constructor;
 	gobject_class->get_property = gdata_documents_entry_get_property;
 	gobject_class->set_property = gdata_documents_entry_set_property;
 	gobject_class->finalize = gdata_documents_entry_finalize;
@@ -104,10 +108,10 @@ gdata_documents_entry_class_init (GDataDocumentsEntryClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_EDITED,
-				g_param_spec_boxed ("edited",
-					"Edited", "The last time the document was edited.",
-					GDATA_TYPE_G_TIME_VAL,
-					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_int64 ("edited",
+	                                                     "Edited", "The last time the document was edited.",
+	                                                     -1, G_MAXINT64, -1,
+	                                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataDocumentsEntry:last-viewed
@@ -117,10 +121,10 @@ gdata_documents_entry_class_init (GDataDocumentsEntryClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_LAST_VIEWED,
-				g_param_spec_boxed ("last-viewed",
-					"Last viewed", "The last time the document was viewed.",
-					GDATA_TYPE_G_TIME_VAL,
-					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_int64 ("last-viewed",
+	                                                     "Last viewed", "The last time the document was viewed.",
+	                                                     -1, G_MAXINT64, -1,
+	                                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataDocumentsEntry:writers-can-invite:
@@ -130,10 +134,10 @@ gdata_documents_entry_class_init (GDataDocumentsEntryClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_WRITERS_CAN_INVITE,
-				g_param_spec_boolean ("writers-can-invite",
-					"Writers can invite?", "Indicates whether writers can invite others to edit.",
-					FALSE,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_boolean ("writers-can-invite",
+	                                                       "Writers can invite?", "Indicates whether writers can invite others to edit.",
+	                                                       FALSE,
+	                                                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataDocumentsEntry:is-deleted:
@@ -144,10 +148,10 @@ gdata_documents_entry_class_init (GDataDocumentsEntryClass *klass)
 	 * Since: 0.5.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_IS_DELETED,
-				g_param_spec_boolean ("is-deleted",
-					"Deleted?", "Indicates whether the document entry has been deleted.",
-					FALSE,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_boolean ("is-deleted",
+	                                                       "Deleted?", "Indicates whether the document entry has been deleted.",
+	                                                       FALSE,
+	                                                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataDocumentsEntry:document-id
@@ -157,10 +161,10 @@ gdata_documents_entry_class_init (GDataDocumentsEntryClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_DOCUMENT_ID,
-				g_param_spec_string ("document-id",
-					"Document ID", "The document ID of the document.",
-					NULL,
-					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("document-id",
+	                                                      "Document ID", "The document ID of the document.",
+	                                                      NULL,
+	                                                      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataDocumentsEntry:last-modified-by
@@ -170,26 +174,17 @@ gdata_documents_entry_class_init (GDataDocumentsEntryClass *klass)
 	 * Since: 0.4.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_LAST_MODIFIED_BY,
-				g_param_spec_object ("last-modified-by",
-					"Last modified by", "Indicates the author of the last modification.",
-					GDATA_TYPE_AUTHOR,
-					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_object ("last-modified-by",
+	                                                      "Last modified by", "Indicates the author of the last modification.",
+	                                                      GDATA_TYPE_AUTHOR,
+	                                                      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 }
 
-static void
-gdata_documents_entry_init (GDataDocumentsEntry *self)
-{
-	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GDATA_TYPE_DOCUMENTS_ENTRY, GDataDocumentsEntryPrivate);
-
-	/* Initialise the edited properties to the current time */
-	g_get_current_time (&(self->priv->edited));
-}
-
 static gboolean
 is_owner_rule (GDataAccessRule *rule)
 {
-	return (strcmp (gdata_access_rule_get_role (rule), "owner") == 0) ? TRUE : FALSE;
+	return (strcmp (gdata_access_rule_get_role (rule), GDATA_DOCUMENTS_ACCESS_ROLE_OWNER) == 0) ? TRUE : FALSE;
 }
 
 static void
@@ -198,42 +193,124 @@ gdata_documents_entry_access_handler_init (GDataAccessHandlerIface *iface)
 	iface->is_owner_rule = is_owner_rule;
 }
 
+static void
+gdata_documents_entry_init (GDataDocumentsEntry *self)
+{
+	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GDATA_TYPE_DOCUMENTS_ENTRY, GDataDocumentsEntryPrivate);
+	self->priv->edited = -1;
+	self->priv->last_viewed = -1;
+}
+
+static GObject *
+gdata_documents_entry_constructor (GType type, guint n_construct_params, GObjectConstructParam *construct_params)
+{
+	GObject *object;
+
+	/* Chain up to the parent class */
+	object = G_OBJECT_CLASS (gdata_documents_entry_parent_class)->constructor (type, n_construct_params, construct_params);
+
+	/* We can't create these in init, or they would collide with the group and control created when parsing the XML */
+	if (_gdata_parsable_is_constructed_from_xml (GDATA_PARSABLE (object)) == FALSE) {
+		GDataDocumentsEntryPrivate *priv = GDATA_DOCUMENTS_ENTRY (object)->priv;
+		GTimeVal time_val;
+
+		/* This can't be put in the init function of #GDataDocumentsEntry, as it would then be called even for entries parsed from XML from
+		 * the server, which would break duplicate element detection for the app:edited element. */
+		g_get_current_time (&time_val);
+		priv->edited = time_val.tv_sec;
+	}
+
+	return object;
+}
+
+static void
+gdata_entry_dispose (GObject *object)
+{
+	GDataDocumentsEntryPrivate *priv = GDATA_DOCUMENTS_ENTRY (object)->priv;
+
+	if (priv->last_modified_by != NULL)
+		g_object_unref (priv->last_modified_by);
+	priv->last_modified_by = NULL;
+
+	/* Chain up to the parent class */
+	G_OBJECT_CLASS (gdata_documents_entry_parent_class)->dispose (object);
+}
+
+static void
+gdata_documents_entry_finalize (GObject *object)
+{
+	GDataDocumentsEntryPrivate *priv = GDATA_DOCUMENTS_ENTRY (object)->priv;
+
+	g_free (priv->document_id);
+
+	/* Chain up to the parent class */
+	G_OBJECT_CLASS (gdata_documents_entry_parent_class)->finalize (object);
+}
+
+static void
+gdata_documents_entry_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec)
+{
+	GDataDocumentsEntryPrivate *priv = GDATA_DOCUMENTS_ENTRY (object)->priv;
+
+	switch (property_id) {
+		case PROP_DOCUMENT_ID:
+			g_value_set_string (value, priv->document_id);
+			break;
+		case PROP_WRITERS_CAN_INVITE:
+			g_value_set_boolean (value, priv->writers_can_invite);
+			break;
+		case PROP_IS_DELETED:
+			g_value_set_boolean (value, priv->is_deleted);
+			break;
+		case PROP_EDITED:
+			g_value_set_int64 (value, priv->edited);
+			break;
+		case PROP_LAST_VIEWED:
+			g_value_set_int64 (value, priv->last_viewed);
+			break;
+		case PROP_LAST_MODIFIED_BY:
+			g_value_set_object (value, priv->last_modified_by);
+			break;
+		default:
+			/* We don't have any other property... */
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+			break;
+	}
+}
+
+static void
+gdata_documents_entry_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec)
+{
+	GDataDocumentsEntry *self = GDATA_DOCUMENTS_ENTRY (object);
+
+	switch (property_id) {
+		case PROP_WRITERS_CAN_INVITE:
+			gdata_documents_entry_set_writers_can_invite (self, g_value_get_boolean (value));
+			break;
+		default:
+			/* We don't have any other property... */
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+			break;
+	}
+}
+
 static gboolean
 parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_data, GError **error)
 {
+	gboolean success;
 	GDataDocumentsEntry *self = GDATA_DOCUMENTS_ENTRY (parsable);
 
 	if (gdata_parser_is_namespace (node, "http://www.w3.org/2007/app") == TRUE &&
-	    xmlStrcmp (node->name, (xmlChar*) "edited") == 0) {
-		xmlChar *edited = xmlNodeListGetString (doc, node->children, TRUE);
-		if (g_time_val_from_iso8601 ((gchar*) edited, &(self->priv->edited)) == FALSE) {
-			gdata_parser_error_not_iso8601_format (node, (gchar*) edited, error);
-			xmlFree (edited);
-			return FALSE;
-		}
-		xmlFree (edited);
-	} else if (gdata_parser_is_namespace (node, "http://schemas.google.com/docs/2007") == TRUE &&
-	           xmlStrcmp (node->name, (xmlChar*) "writersCanInvite") ==  0) {
-		xmlChar *writers_can_invite = xmlGetProp (node, (xmlChar*) "value");
-		if (xmlStrcmp (writers_can_invite, (xmlChar*) "true") == 0) {
-			self->priv->writers_can_invite = TRUE;
-		} else if (xmlStrcmp (writers_can_invite, (xmlChar*) "false") == 0) {
-			self->priv->writers_can_invite = FALSE;
-		} else {
-			gdata_parser_error_unknown_property_value (node, "value", (gchar*) writers_can_invite, error);
-			xmlFree (writers_can_invite);
-			return FALSE;
-		}
-		xmlFree (writers_can_invite);
+	    gdata_parser_int64_from_element (node, "edited", P_REQUIRED | P_NO_DUPES, &(self->priv->edited), &success, error) == TRUE) {
+		return success;
 	} else if (gdata_parser_is_namespace (node, "http://schemas.google.com/g/2005") == TRUE) {
-		if (xmlStrcmp (node->name, (xmlChar*) "lastViewed") == 0) {
-			xmlChar *last_viewed = xmlNodeListGetString (doc, node->children, TRUE);
-			if (g_time_val_from_iso8601 ((gchar*) last_viewed, &(self->priv->last_viewed)) == FALSE) {
-				gdata_parser_error_not_iso8601_format (node, (gchar*) last_viewed, error);
-				xmlFree (last_viewed);
-				return FALSE;
-			}
-			xmlFree (last_viewed);
+		if (gdata_parser_int64_from_element (node, "lastViewed", P_REQUIRED | P_NO_DUPES,
+		                                     &(self->priv->last_viewed), &success, error) == TRUE ||
+		    gdata_parser_object_from_element_setter (node, "feedLink", P_REQUIRED, GDATA_TYPE_LINK,
+		                                             gdata_entry_add_link, self,  &success, error) == TRUE ||
+		    gdata_parser_object_from_element (node, "lastModifiedBy", P_REQUIRED, GDATA_TYPE_AUTHOR,
+		                                      &(self->priv->last_modified_by), &success, error) == TRUE) {
+			return success;
 		} else if (xmlStrcmp (node->name, (xmlChar*) "deleted") ==  0) {
 			/* <gd:deleted> */
 			/* Note that it doesn't have any parameters, so we unconditionally set priv->is_deleted to TRUE */
@@ -261,20 +338,13 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 
 			self->priv->document_id = g_strdup (document_id_parts[1]);
 			g_strfreev (document_id_parts);
-		} else if (xmlStrcmp (node->name, (xmlChar*) "feedLink") ==  0) {
-			GDataLink *link = GDATA_LINK (_gdata_parsable_new_from_xml_node (GDATA_TYPE_LINK, doc, node, NULL, error));
-			if (link == NULL)
-				return FALSE;
-			gdata_entry_add_link (GDATA_ENTRY (self), link);
-			g_object_unref (link);
-		} else if (xmlStrcmp (node->name, (xmlChar*) "lastModifiedBy") ==  0) {
-			GDataAuthor *last_modified_by = GDATA_AUTHOR (_gdata_parsable_new_from_xml_node (GDATA_TYPE_AUTHOR, doc, node, NULL, error));
-			if (last_modified_by == NULL)
-				return FALSE;
-			self->priv->last_modified_by = last_modified_by;
 		} else {
 			return GDATA_PARSABLE_CLASS (gdata_documents_entry_parent_class)->parse_xml (parsable, doc, node, user_data, error);
 		}
+	} else if (gdata_parser_is_namespace (node, "http://schemas.google.com/docs/2007") == TRUE &&
+	           xmlStrcmp (node->name, (xmlChar*) "writersCanInvite") ==  0) {
+		if (gdata_parser_boolean_from_property (node, "value", &(self->priv->writers_can_invite), -1, error) == FALSE)
+			return FALSE;
 	} else {
 		return GDATA_PARSABLE_CLASS (gdata_documents_entry_parent_class)->parse_xml (parsable, doc, node, user_data, error);
 	}
@@ -282,77 +352,6 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 	return TRUE;
 }
 
-static void
-gdata_documents_entry_finalize (GObject *object)
-{
-	GDataDocumentsEntryPrivate *priv = GDATA_DOCUMENTS_ENTRY_GET_PRIVATE (object);
-
-	g_free (priv->document_id);
-
-	/* Chain up to the parent class */
-	G_OBJECT_CLASS (gdata_documents_entry_parent_class)->finalize (object);
-}
-
-static void
-gdata_entry_dispose (GObject *object)
-{
-	GDataDocumentsEntryPrivate *priv = GDATA_DOCUMENTS_ENTRY_GET_PRIVATE (object);
-
-	if (priv->last_modified_by != NULL)
-		g_object_unref (priv->last_modified_by);
-	priv->last_modified_by = NULL;
-
-	/* Chain up to the parent class */
-	G_OBJECT_CLASS (gdata_documents_entry_parent_class)->dispose (object);
-}
-
-static void
-gdata_documents_entry_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec)
-{
-	GDataDocumentsEntryPrivate *priv = GDATA_DOCUMENTS_ENTRY_GET_PRIVATE (object);
-
-	switch (property_id) {
-		case PROP_DOCUMENT_ID:
-			g_value_set_string (value, priv->document_id);
-			break;
-		case PROP_WRITERS_CAN_INVITE:
-			g_value_set_boolean (value, priv->writers_can_invite);
-			break;
-		case PROP_IS_DELETED:
-			g_value_set_boolean (value, priv->is_deleted);
-			break;
-		case PROP_EDITED:
-			g_value_set_boxed (value, &(priv->edited));
-			break;
-		case PROP_LAST_VIEWED:
-			g_value_set_boxed (value, &(priv->last_viewed));
-			break;
-		case PROP_LAST_MODIFIED_BY:
-			g_value_set_object (value, priv->last_modified_by);
-			break;
-		default:
-			/* We don't have any other property... */
-			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
-			break;
-	}
-}
-
-static void
-gdata_documents_entry_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec)
-{
-	GDataDocumentsEntry *self = GDATA_DOCUMENTS_ENTRY (object);
-
-	switch (property_id) {
-		case PROP_WRITERS_CAN_INVITE:
-			gdata_documents_entry_set_writers_can_invite (self, g_value_get_boolean (value));
-			break;
-		default:
-			/* We don't have any other property... */
-			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
-			break;
-	}
-}
-
 static void
 get_xml (GDataParsable *parsable, GString *xml_string)
 {
@@ -381,47 +380,45 @@ get_namespaces (GDataParsable *parsable, GHashTable *namespaces)
 /**
  * gdata_documents_entry_get_edited:
  * @self: a #GDataDocumentsEntry
- * @edited: (out caller-allocates): a #GTimeVal
  *
- * Gets the #GDataDocumentsEntry:edited property and puts it in @edited. If the property is unset,
- * both fields in the #GTimeVal will be set to <code class="literal">0</code>.
+ * Gets the #GDataDocumentsEntry:edited property. If the property is unset, <code class="literal">-1</code> will be returned.
+ *
+ * Return value: the UNIX timestamp for the time the document was last edited, or <code class="literal">-1</code>
  *
  * Since: 0.4.0
  **/
-void
-gdata_documents_entry_get_edited (GDataDocumentsEntry *self, GTimeVal *edited)
+gint64
+gdata_documents_entry_get_edited (GDataDocumentsEntry *self)
 {
-	g_return_if_fail (GDATA_IS_DOCUMENTS_ENTRY (self));
-	g_return_if_fail (edited != NULL);
-	*edited = self->priv->edited;
+	g_return_val_if_fail (GDATA_IS_DOCUMENTS_ENTRY (self), -1);
+	return self->priv->edited;
 }
 
 /**
  * gdata_documents_entry_get_last_viewed:
  * @self: a #GDataDocumentsEntry
- * @last_viewed: (out caller-allocates): a #GTimeVal
  *
- * Gets the #GDataDocumentsEntry:last-viewed property and puts it in @last_viewed. If the property is unset,
- * both fields in the #GTimeVal will be set to <code class="literal">0</code>.
+ * Gets the #GDataDocumentsEntry:last-viewed property. If the property is unset, <code class="literal">-1</code> will be returned.
+ *
+ * Return value: the UNIX timestamp for the time the document was last viewed, or <code class="literal">-1</code>
  *
  * Since: 0.4.0
  **/
-void
-gdata_documents_entry_get_last_viewed (GDataDocumentsEntry *self, GTimeVal *last_viewed)
+gint64
+gdata_documents_entry_get_last_viewed (GDataDocumentsEntry *self)
 {
-	g_return_if_fail (GDATA_IS_DOCUMENTS_ENTRY (self));
-	g_return_if_fail (last_viewed != NULL);
-	*last_viewed = self->priv->last_viewed;
+	g_return_val_if_fail (GDATA_IS_DOCUMENTS_ENTRY (self), -1);
+	return self->priv->last_viewed;
 }
 
 /**
  * gdata_documents_entry_get_path:
  * @self: a #GDataDocumentsEntry
  *
- * Builds a path for the #GDataDocumentsEntry, starting from a root node and traversing the folders containing the document, then
- * ending with the document's ID.
+ * Builds a path for the #GDataDocumentsEntry, starting from a root node and traversing the folders containing the document, then ending with the
+ * document's ID.
  *
- * An example path would be: "/folder_id1/folder_id2/document_id".
+ * An example path would be: <literal>/folder_id1/folder_id2/document_id</literal>.
  *
  * Note: the path is based on the entry/document IDs of the folders (#GDataEntry:id) and document (#GDataDocumentsEntry:document-id),
  * and not the entries' human-readable names (#GDataEntry:title).
@@ -565,102 +562,3 @@ gdata_documents_entry_is_deleted (GDataDocumentsEntry *self)
 	g_return_val_if_fail (GDATA_IS_DOCUMENTS_ENTRY (self), FALSE);
 	return self->priv->is_deleted;
 }
-
-static void
-notify_content_type_cb (GDataDownloadStream *download_stream, GParamSpec *pspec, gchar **content_type)
-{
-	*content_type = g_strdup (gdata_download_stream_get_content_type (download_stream));
-}
-
-/*
- * _gdata_documents_entry_download_document:
- * @self: a #GDataDocumentsEntry
- * @service: an authenticated #GDataDocumentsService
- * @content_type: (out callee-allocates) (transfer full) (allow-none): return location for the document's content type, or %NULL; free with g_free()
- * @download_uri: the URI to download the document
- * @destination_file: the #GFile into which the document file should be saved
- * @file_extension: the extension with which to save the downloaded file
- * @replace_file_if_exists: %TRUE if you want to replace the file if it exists, %FALSE otherwise
- * @cancellable: optional #GCancellable object, or %NULL
- * @error: a #GError, or %NULL
- *
- * Downloads and returns the actual file which comprises the document here. If the document doesn't exist,
- * the downloaded document will be an HTML file containing the error explanation.
- * TODO: Is that still true?
- *
- * If @cancellable is not %NULL, then the operation can be cancelled by triggering the @cancellable object from another thread.
- * If the operation was cancelled, the error %G_IO_ERROR_CANCELLED will be returned.
- *
- * If @replace_file_if_exists is set to %FALSE and the destination file already exists, a %G_IO_ERROR_EXISTS will be returned.
- *
- * If @service isn't authenticated, a %GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED is returned.
- *
- * If there is an error downloading the document, a %GDATA_SERVICE_ERROR_PROTOCOL_ERROR error will be returned.
- *
- * If @destination_file is a directory, the file will be downloaded to this directory with the #GDataEntry:title and 
- * the appropriate extension as its filename.
- *
- * Return value: a #GFile pointing to the downloaded document, or %NULL; unref with g_object_unref()
- *
- * Since: 0.4.0
- */
-GFile *
-_gdata_documents_entry_download_document (GDataDocumentsEntry *self, GDataService *service, gchar **content_type, const gchar *src_uri,
-					  GFile *destination_file, const gchar *file_extension, gboolean replace_file_if_exists,
-					  GCancellable *cancellable, GError **error)
-{
-	const gchar *document_title;
-	gchar *default_filename;
-	GFileOutputStream *dest_stream;
-	GInputStream *src_stream;
-	GFile *actual_file = NULL;
-	GError *child_error = NULL;
-	gulong content_type_signal = 0;
-
-	/* TODO: async version */
-	g_return_val_if_fail (GDATA_IS_DOCUMENTS_ENTRY (self), NULL);
-	g_return_val_if_fail (GDATA_IS_SERVICE (service), NULL);
-	g_return_val_if_fail (src_uri != NULL, NULL);
-	g_return_val_if_fail (G_IS_FILE (destination_file), NULL);
-	g_return_val_if_fail (file_extension != NULL, NULL);
-	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
-	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
-
-	/* Ensure we're authenticated first */
-	if (gdata_service_is_authenticated (GDATA_SERVICE (service)) == FALSE) {
-		g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
-				     _("You must be authenticated to download documents."));
-		return NULL;
-	}
-
-	/* Determine a default filename based on the document's title */
-	document_title = gdata_entry_get_title (GDATA_ENTRY (self));
-	default_filename = g_strdup_printf ("%s.%s", document_title, file_extension);
-
-	dest_stream = _gdata_download_stream_find_destination (default_filename, destination_file, &actual_file, replace_file_if_exists, cancellable, error);
-	g_free (default_filename);
-
-	if (dest_stream == NULL)
-		return NULL;
-
-	/* Synchronously splice the data from the download stream to the file stream (network -> disk) */
-	src_stream = gdata_download_stream_new (GDATA_SERVICE (service), src_uri);
-	if (content_type != NULL)
-		content_type_signal = g_signal_connect (src_stream, "notify::content-type", (GCallback) notify_content_type_cb, content_type);
-
-	g_output_stream_splice (G_OUTPUT_STREAM (dest_stream), src_stream,
-				G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE | G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET, cancellable, &child_error);
-
-	if (content_type_signal != 0)
-		g_signal_handler_disconnect (src_stream, content_type_signal);
-
-	g_object_unref (src_stream);
-	g_object_unref (dest_stream);
-	if (child_error != NULL) {
-		g_object_unref (actual_file);
-		g_propagate_error (error, child_error);
-		return NULL;
-	}
-
-	return actual_file;
-}
