@@ -25,8 +25,8 @@
  *
  * #GDataYouTubeControl represents a "control" element from the
  * <ulink type="http" url="http://www.atomenabled.org/developers/protocol/#appControl">Atom Publishing Protocol specification</ulink>,
- * with support for the YouTube-specific <ulink type="http" url="http://code.google.com/apis/youtube/2.0/reference.html#youtube_data_api_tag_yt:state">
- * "state" element</ulink>.
+ * with support for the YouTube-specific
+ * <ulink type="http" url="http://code.google.com/apis/youtube/2.0/reference.html#youtube_data_api_tag_yt:state">"state" element</ulink>.
  *
  * It is private API, since it would be unnecessary and confusing to expose #GDataYouTubeControl itself. There is the option
  * of breaking this up into #GDataAPPControl and extending #GDataYouTubeControl classes in the future, if more entry types implement
@@ -55,7 +55,6 @@ struct _GDataYouTubeControlPrivate {
 };
 
 G_DEFINE_TYPE (GDataYouTubeControl, gdata_youtube_control, GDATA_TYPE_PARSABLE)
-#define GDATA_YOUTUBE_CONTROL_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GDATA_TYPE_YOUTUBE_CONTROL, GDataYouTubeControlPrivate))
 
 static void
 gdata_youtube_control_class_init (GDataYouTubeControlClass *klass)
@@ -96,6 +95,7 @@ gdata_youtube_control_dispose (GObject *object)
 static gboolean
 parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_data, GError **error)
 {
+	gboolean success;
 	GDataYouTubeControl *self = GDATA_YOUTUBE_CONTROL (parsable);
 
 	if (gdata_parser_is_namespace (node, "http://www.w3.org/2007/app") == TRUE &&
@@ -108,18 +108,9 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 			self->priv->is_draft = TRUE;
 		xmlFree (draft);
 	} else if (gdata_parser_is_namespace (node, "http://gdata.youtube.com/schemas/2007") == TRUE &&
-	           xmlStrcmp (node->name, (xmlChar*) "state") == 0) {
-		/* yt:state */
-		GDataYouTubeState *state = GDATA_YOUTUBE_STATE (_gdata_parsable_new_from_xml_node (GDATA_TYPE_YOUTUBE_STATE, doc, node, NULL, error));
-		if (state == NULL)
-			return FALSE;
-
-		if (self->priv->state != NULL) {
-			g_object_unref (state);
-			return gdata_parser_error_duplicate_element (node, error);
-		}
-
-		self->priv->state = state;
+	           gdata_parser_object_from_element (node, "state", P_REQUIRED | P_NO_DUPES, GDATA_TYPE_YOUTUBE_STATE,
+	                                             &(self->priv->state), &success, error) == TRUE) {
+		return success;
 	} else {
 		return GDATA_PARSABLE_CLASS (gdata_youtube_control_parent_class)->parse_xml (parsable, doc, node, user_data, error);
 	}
