@@ -48,6 +48,8 @@ test_authentication (void)
 	g_assert (gdata_service_is_authenticated (GDATA_SERVICE (service)) == TRUE);
 	g_assert_cmpstr (gdata_service_get_username (GDATA_SERVICE (service)), ==, DOCUMENTS_USERNAME);
 	g_assert_cmpstr (gdata_service_get_password (GDATA_SERVICE (service)), ==, PASSWORD);
+
+	g_object_unref (service);
 }
 
 static void
@@ -90,6 +92,7 @@ test_remove_all_documents_and_folders (gconstpointer service)
 		g_clear_error (&error);
 	}
 
+	g_object_unref (query);
 	g_object_unref (feed);
 }
 
@@ -111,6 +114,7 @@ test_query_all_documents_with_folder (gconstpointer service)
 
 	g_clear_error (&error);
 	g_object_unref (feed);
+	g_object_unref (query);
 }
 
 static void
@@ -358,7 +362,7 @@ test_upload_file_metadata_in_new_folder (gconstpointer service)
 static void
 test_update_metadata (gconstpointer service)
 {
-	GDataDocumentsEntry *document, *new_document, *updated_document;
+	GDataDocumentsEntry *document, *new_document, *new_document2, *updated_document;
 	GError *error = NULL;
 
 	g_assert (service != NULL);
@@ -371,24 +375,34 @@ test_update_metadata (gconstpointer service)
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_DOCUMENTS_TEXT (new_document));
 
+	/* HACK: Query for the new document, as Google's servers appear to modify it behind our back if we don't upload both metadata and data when
+	 * creating the document: http://code.google.com/a/google.com/p/apps-api-issues/issues/detail?id=2337. We have to wait a few seconds before
+	 * trying this to allow the various Google servers to catch up with each other. */
+	g_usleep (5 * G_USEC_PER_SEC);
+	new_document2 = GDATA_DOCUMENTS_ENTRY (gdata_service_query_single_entry (GDATA_SERVICE (service),
+	                                                                         gdata_entry_get_id (GDATA_ENTRY (new_document)), NULL,
+	                                                                         GDATA_TYPE_DOCUMENTS_TEXT, NULL, &error));
+
+	g_object_unref (new_document);
+
 	/* Change the title */
-	gdata_entry_set_title (GDATA_ENTRY (document), "update_metadata_updated_title");
+	gdata_entry_set_title (GDATA_ENTRY (new_document2), "update_metadata_updated_title");
 
 	/* Update the document */
-	updated_document = gdata_documents_service_update_document (GDATA_DOCUMENTS_SERVICE (service), new_document, NULL, NULL, &error);
+	updated_document = gdata_documents_service_update_document (GDATA_DOCUMENTS_SERVICE (service), new_document2, NULL, NULL, &error);
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_DOCUMENTS_TEXT (updated_document));
 
 	g_clear_error (&error);
 	g_object_unref (document);
-	g_object_unref (new_document);
+	g_object_unref (new_document2);
 	g_object_unref (updated_document);
 }
 
 static void
 test_update_metadata_file (gconstpointer service)
 {
-	GDataDocumentsEntry *document, *new_document, *updated_document;
+	GDataDocumentsEntry *document, *new_document, *new_document2, *updated_document;
 	GFile *document_file, *updated_document_file;
 	GError *error = NULL;
 
@@ -405,17 +419,27 @@ test_update_metadata_file (gconstpointer service)
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_DOCUMENTS_TEXT (new_document));
 
+	/* HACK: Query for the new document, as Google's servers appear to modify it behind our back if we don't upload both metadata and data when
+	 * creating the document: http://code.google.com/a/google.com/p/apps-api-issues/issues/detail?id=2337. We have to wait a few seconds before
+	 * trying this to allow the various Google servers to catch up with each other. */
+	g_usleep (5 * G_USEC_PER_SEC);
+	new_document2 = GDATA_DOCUMENTS_ENTRY (gdata_service_query_single_entry (GDATA_SERVICE (service),
+	                                                                         gdata_entry_get_id (GDATA_ENTRY (new_document)), NULL,
+	                                                                         GDATA_TYPE_DOCUMENTS_TEXT, NULL, &error));
+
+	g_object_unref (new_document);
+
 	/* Change the title of the document */
-	gdata_entry_set_title (GDATA_ENTRY (new_document), "update_metadata_file_updated_title");
+	gdata_entry_set_title (GDATA_ENTRY (new_document2), "update_metadata_file_updated_title");
 
 	/* Update the document */
-	updated_document = gdata_documents_service_update_document (GDATA_DOCUMENTS_SERVICE (service), new_document, updated_document_file, NULL, &error);
+	updated_document = gdata_documents_service_update_document (GDATA_DOCUMENTS_SERVICE (service), new_document2, updated_document_file, NULL, &error);
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_DOCUMENTS_TEXT (updated_document));
 
 	g_clear_error (&error);
 	g_object_unref (document);
-	g_object_unref (new_document);
+	g_object_unref (new_document2);
 	g_object_unref (updated_document);
 	g_object_unref (document_file);
 }
@@ -423,7 +447,7 @@ test_update_metadata_file (gconstpointer service)
 static void
 test_update_file (gconstpointer service)
 {
-	GDataDocumentsEntry *new_document, *updated_document;
+	GDataDocumentsEntry *new_document, *new_document2, *updated_document;
 	GFile *document_file, *updated_document_file;
 	GError *error = NULL;
 
@@ -437,15 +461,25 @@ test_update_file (gconstpointer service)
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_DOCUMENTS_PRESENTATION (new_document));
 
+	/* HACK: Query for the new document, as Google's servers appear to modify it behind our back if we don't upload both metadata and data when
+	 * creating the document: http://code.google.com/a/google.com/p/apps-api-issues/issues/detail?id=2337. We have to wait a few seconds before
+	 * trying this to allow the various Google servers to catch up with each other. */
+	g_usleep (5 * G_USEC_PER_SEC);
+	new_document2 = GDATA_DOCUMENTS_ENTRY (gdata_service_query_single_entry (GDATA_SERVICE (service),
+	                                                                         gdata_entry_get_id (GDATA_ENTRY (new_document)), NULL,
+	                                                                         GDATA_TYPE_DOCUMENTS_PRESENTATION, NULL, &error));
+
+	g_object_unref (new_document);
+
 	/* Update the document */
-	updated_document = gdata_documents_service_update_document (GDATA_DOCUMENTS_SERVICE (service), new_document, document_file, NULL, &error);
+	updated_document = gdata_documents_service_update_document (GDATA_DOCUMENTS_SERVICE (service), new_document2, document_file, NULL, &error);
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_DOCUMENTS_PRESENTATION (updated_document));
 	g_clear_error (&error);
 
 	g_object_unref (document_file);
 	g_object_unref (updated_document_file);
-	g_object_unref (new_document);
+	g_object_unref (new_document2);
 	g_object_unref (updated_document);
 }
 
@@ -768,9 +802,10 @@ test_batch_async (BatchAsyncData *data, gconstpointer service)
 	main_loop = g_main_loop_new (NULL, TRUE);
 
 	gdata_batch_operation_run_async (operation, NULL, (GAsyncReadyCallback) test_batch_async_cb, main_loop);
-
 	g_main_loop_run (main_loop);
+
 	g_main_loop_unref (main_loop);
+	g_object_unref (operation);
 }
 
 static void
@@ -807,6 +842,7 @@ test_batch_async_cancellation (BatchAsyncData *data, gconstpointer service)
 	g_main_loop_run (main_loop);
 	g_main_loop_unref (main_loop);
 	g_object_unref (cancellable);
+	g_object_unref (operation);
 }
 
 static void
@@ -821,10 +857,8 @@ teardown_batch_async (BatchAsyncData *data, gconstpointer service)
 	g_assert_no_error (error);
 	g_clear_error (&error);
 
-	/* Delete the document */
-	g_assert (gdata_service_delete_entry (GDATA_SERVICE (service), document, NULL, &error) == TRUE);
-	g_assert_no_error (error);
-	g_clear_error (&error);
+	/* Delete the document (we don't care if this fails) */
+	gdata_service_delete_entry (GDATA_SERVICE (service), document, NULL, NULL);
 
 	g_object_unref (data->new_doc);
 	g_object_unref (document);
